import { IReadonlyRange2d, Range2d } from "../range2d/range2d.js";
import { TTypedArray } from "../../t-typed-array.js";
import { ITypedArrayTupleFactory } from "../../i-typed-array-tuple-factory.js";
import { TTypedArrayCtor } from "../../t-typed-array-ctor.js";
import { Mat2 } from "../../mat2/mat2.js";
import { IReadonlyMat3 } from "../../mat3/mat3.js";
import { TPickExcept } from "../../../../typescript/t-pick-except.js";
import { TTypedArrayTupleMutativeMethods } from "../../a-typed-array-tuple.js";
/**
 * @public
 */
export type TMargin2dCtorArgs = [left: number, right: number, top: number, bottom: number];
/**
 * @public
 * Constructor for {@link Margin2d}.
 */
export interface IMargin2dCtor<TArray extends TTypedArray> {
    /**
     * The size in bytes of each element in the array.
     */
    readonly BYTES_PER_ELEMENT: number;
    readonly prototype: Margin2d<TArray>;
    readonly factory: ITypedArrayTupleFactory<Margin2d<TArray>, TMargin2dCtorArgs>;
    new (): Margin2d<TArray>;
}
/**
 * @public
 * {@link Margin2d}.
 */
export interface IReadonlyMargin2d<TArray extends TTypedArray> extends TPickExcept<Readonly<Margin2d<TArray>>, "setLeft" | "setRight" | "setTop" | "setBottom" | "update" | "copyFromBuffer" | TTypedArrayTupleMutativeMethods> {
}
/**
 * @public
 * 2x2 matrix representing margins on a rectangle.
 *
 * @remarks
 * See static properties for constructors. Instances are not an extension of this class, but of the static members.
 */
export declare abstract class Margin2d<TArray extends TTypedArray> extends Mat2<TArray> {
    static f64: IMargin2dCtor<Float64Array>;
    static f32: IMargin2dCtor<Float32Array>;
    static u32: IMargin2dCtor<Uint32Array>;
    static i32: IMargin2dCtor<Int32Array>;
    static u16: IMargin2dCtor<Uint16Array>;
    static i16: IMargin2dCtor<Int16Array>;
    static u8c: IMargin2dCtor<Uint8ClampedArray>;
    static u8: IMargin2dCtor<Uint8Array>;
    static i8: IMargin2dCtor<Int8Array>;
    static getCtor<TCtor extends TTypedArrayCtor>(ctor: TCtor): IMargin2dCtor<InstanceType<TCtor>>;
    protected static constructors: Map<TTypedArrayCtor, Function>;
    ["constructor"]: IMargin2dCtor<TArray>;
    /**
     * left
     */
    0: number;
    /**
     * right
     */
    1: number;
    /**
     * top
     */
    2: number;
    /**
     * bottom
     */
    3: number;
    getLeft(): number;
    getRight(): number;
    getTop(): number;
    getBottom(): number;
    setLeft(_value: number): void;
    setRight(_value: number): void;
    setTop(_value: number): void;
    setBottom(_value: number): void;
    update(..._args: TMargin2dCtorArgs): void;
    sumX(): number;
    sumY(): number;
    getInnerRange<TResult extends TTypedArray = TArray>(_range: IReadonlyRange2d<TTypedArray>, _result?: Range2d<TResult>): Range2d<TResult>;
    /**
     * Applies `Mat3.getTransformedXLength` `Mat3.getTransformedYLength` with a minimum of 0 and a maximum of
     * whatever the margin is.
     */
    mat3TransformLength<TResult extends TTypedArray = TArray>(_mat: IReadonlyMat3<TTypedArray>, _writeTo?: Margin2d<TResult>): Margin2d<TResult>;
    /**
     * If endianness is not supplied the platform's endianness will be used.
     */
    copyFromBuffer(_memoryDataView: DataView, _pointer: number, _littleEndian?: boolean): void;
    /**
     * If endianness is not supplied the platform's endianness will be used.
     */
    copyToBuffer(_memoryDataView: DataView, _pointer: number, _littleEndian?: boolean): void;
    /**
     * Although the typed array tuples extend a typed array, they are not structurally compatible.
     * This function returns the argument passed without modification but cast as the underlying storage type, e.g. Float32Array.
     */
    castToBaseType(): TArray;
    TTypeGuardAMargin2d: true;
}
/**
 * @public
 * Float32 {@link Margin2d}.
 */
export type TF32Margin2d = Margin2d<Float32Array>;
/**
 * @public
 * Float64 {@link Margin2d}.
 */
export type TF64Margin2d = Margin2d<Float64Array>;
