"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DebugSharedObjectLifeCycleChecker = void 0;
const _debug_js_1 = require("./_debug.js");
const debug_pointer_js_1 = require("./debug-pointer.js");
const _production_js_1 = require("../production/_production.js");
/**
 * When a javascript object is garbage collected, check if the corresponding WASM object has been freed.
 */
class DebugSharedObjectLifeCycleChecker {
    constructor() {
        this.finalizationRegistry = new FinalizationRegistry((debugPointer) => {
            if (this.debugPointers.has(debugPointer)) {
                // must have failed to clean up for the pointer to still be present
                if (debugPointer.isStatic) {
                    // the wasm memory hasn't been cleaned up
                    // the js object may leak in production too (resize listener), but we don't know
                    // fixing one fixes the other
                    const message = [
                        "Finalization registry found unreleased wasm object allocated by:",
                        debugPointer.toString(),
                    ].join("\n");
                    _debug_js_1._Debug.error(message);
                }
                else {
                    // wasm memory never gets cleaned up (static)
                    // the js object will leak in production (resize listener is a strong reference in that case)
                    _debug_js_1._Debug.error([
                        "Leaked shared static object on the JS side:",
                        debugPointer.toString(),
                    ].join("\n"));
                }
            }
        });
        this.owningPointers = new Map();
        this.sharedObjToPtr = new WeakMap();
        this.debugPointers = new Set();
    }
    registerFinalizationCheck(sharedObject) {
        const instanceOwningPtr = this.owningPointers.get(sharedObject.getPtr());
        if (instanceOwningPtr != null) {
            _debug_js_1._Debug.error([
                "Address has already been claimed by:",
                instanceOwningPtr.toString(),
            ].join("\n"));
        }
        const debugPointer = new debug_pointer_js_1.DebugPointer(sharedObject.getPtr(), sharedObject.constructor, sharedObject.isStatic);
        this.debugPointers.add(debugPointer);
        this.sharedObjToPtr.set(sharedObject, debugPointer);
        if (!sharedObject.isStatic) {
            this.owningPointers.set(debugPointer.pointer, debugPointer);
        }
        this.finalizationRegistry.register(sharedObject, debugPointer);
    }
    markReadyForFinalize(sharedObject) {
        if (!sharedObject.isStatic) {
            this.owningPointers.delete(sharedObject.getPtr());
        }
        const pointer = this.sharedObjToPtr.get(sharedObject);
        if (pointer == null) {
            throw _production_js_1._Production.createError("expected to find pointer");
        }
        this.debugPointers.delete(pointer);
    }
}
exports.DebugSharedObjectLifeCycleChecker = DebugSharedObjectLifeCycleChecker;
//# sourceMappingURL=debug-shared-object-life-cycle-checker.js.map