"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SharedArray = void 0;
const reference_counted_ptr_js_1 = require("../util/reference-counted-ptr.js");
const _debug_js_1 = require("../../debug/_debug.js");
const debug_protected_view_js_1 = require("../../debug/debug-protected-view.js");
const _production_js_1 = require("../../production/_production.js");
const null_pointer_js_1 = require("../emscripten/null-pointer.js");
const debug_shared_object_checks_js_1 = require("../util/debug-shared-object-checks.js");
/**
 * @public
 * Typed array shared between wasm and javascript.
 */
class SharedArray {
    static createOneF32(wrapper, bindToReference, length, clearMemory = false, allocationFailThrows = true) {
        return SharedArray.createOne("f32SharedArray", wrapper, bindToReference, Float32Array, length, clearMemory, allocationFailThrows);
    }
    static createOneF64(wrapper, bindToReference, length, clearMemory = false, allocationFailThrows = true) {
        return SharedArray.createOne("f64SharedArray", wrapper, bindToReference, Float64Array, length, clearMemory, allocationFailThrows);
    }
    static createOne(prefix, wrapper, bindToReference, ctor, length, clearMemory = false, allocationFailThrows = true) {
        const ptr = SharedArray.allocateMemory(prefix, wrapper, length, clearMemory, allocationFailThrows);
        if (ptr !== null_pointer_js_1.nullPointer) {
            const sa = new SharedArray(prefix, ctor, wrapper, length, ptr);
            bindToReference === null || bindToReference === void 0 ? void 0 : bindToReference.linkRef(sa.sharedObject);
            return sa;
        }
        return null;
    }
    static allocateMemory(cMethodPrefix, wrapper, length, clearMemory, allocationFailThrows) {
        const ptr = wrapper.instance[`_${cMethodPrefix}_createOne`](length, clearMemory);
        if (ptr === null_pointer_js_1.nullPointer && allocationFailThrows) {
            throw _production_js_1._Production.createError("Failed to allocate memory for shared array.");
        }
        return ptr;
    }
    getInstance() {
        if (_BUILD.DEBUG) {
            _debug_js_1._Debug.assert(!this.sharedObject.getIsDestroyed(), "access violation - object has been destroyed");
            return this.wrapper.debug.protectedViews
                .getValue(this)
                .createProtectedView(this.instance);
        }
        else {
            return this.instance;
        }
    }
    constructor(cMethodPrefix, ctor, wrapper, length, pointer) {
        this.onMemoryResize = () => {
            this.instance = this.createLocalInstance();
        };
        this.sharedObject = new reference_counted_ptr_js_1.ReferenceCountedPtr(false, pointer, wrapper);
        this.sharedObject.registerOnFreeListener(() => this.wrapper.instance[this.cDelete](this.sharedObject.getPtr()));
        this.sharedObject.registerOnFreeListener(wrapper.memoryResize.addTemporaryListener(this));
        this.length = length;
        this.ctor = ctor;
        this.cDelete = `_${cMethodPrefix}_delete`;
        this.cGetArrayAddress = `_${cMethodPrefix}_getArrayAddress`;
        this.wrapper = wrapper;
        this.elementByteSize = ctor.BYTES_PER_ELEMENT;
        _BUILD.DEBUG && _debug_js_1._Debug.runBlock(() => {
            const protectedView = debug_protected_view_js_1.DebugProtectedView.createTypedArrayView(this.wrapper);
            debug_shared_object_checks_js_1.DebugSharedObjectChecks.registerWithCleanup(this, protectedView, "shared array");
        });
        this.instance = this.createLocalInstance();
    }
    createLocalInstance() {
        const arrayPtr = this.wrapper.instance[this.cGetArrayAddress](this.sharedObject.getPtr());
        _BUILD.DEBUG && _debug_js_1._Debug.assert(arrayPtr !== null_pointer_js_1.nullPointer, "failed to get array address");
        return new this.ctor(this.wrapper.memory.buffer, arrayPtr, this.length);
    }
}
exports.SharedArray = SharedArray;
//# sourceMappingURL=shared-array.js.map