"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DebugSharedObjectChecks = void 0;
const _debug_js_1 = require("../../debug/_debug.js");
const number_get_hex_string_js_1 = require("../../number/impl/number-get-hex-string.js");
const string_normalize_null_undefined_to_empty_js_1 = require("../../string/impl/string-normalize-null-undefined-to-empty.js");
/**
 * @public
 * Provides life cycle and access checks for shared objects.
 *
 * @remarks
 * `ProtectedViews` will be invalidated any time a memory resize might occur with standard `_BUILD.DEBUG` set.
 *
 * On garbage collect of the javascript object, the associated WASM pointer is checked to see if it has been disposed of.
 */
class DebugSharedObjectChecks {
    /**
     * Calls register on the shared object but also registers the associated cleanup on release too.
     */
    static registerWithCleanup(instance, protectedView, nameOfInstance) {
        instance.sharedObject.registerOnFreeListener(() => DebugSharedObjectChecks.unregister(protectedView, instance, nameOfInstance));
        return DebugSharedObjectChecks.register(instance, protectedView, nameOfInstance);
    }
    static register(instance, protectedView, nameOfInstance) {
        if (instance.debugOnAllocate == null) {
            instance.debugOnAllocate = () => protectedView.invalidate();
        }
        const debug = protectedView.owningInstance.debug;
        debug.protectedViews.setValue(instance, protectedView);
        debug.sharedObjectLifeCycleChecks.registerFinalizationCheck(instance.sharedObject);
        debug.onAllocate.addListener(instance);
        if (!instance.sharedObject.isStatic) {
            // stringifying the stack would be far too verbose, most debuggers allow expansion of objects...
            const allocationStack = { stack: _debug_js_1._Debug.getStackTrace() };
            const type = instance.sharedObject.isStatic ? "static" : "instance";
            const message = `claimed (${type}) ${nameOfInstance} ${(0, number_get_hex_string_js_1.numberGetHexString)(instance.sharedObject.getPtr())} - ${(0, string_normalize_null_undefined_to_empty_js_1.stringNormalizeNullUndefinedToEmpty)(_debug_js_1._Debug.label)}`;
            _debug_js_1._Debug.verboseLog(["WASM", "MEMORY"], message, allocationStack);
        }
        return protectedView;
    }
    static unregister(protectedView, instance, nameOfInstance) {
        const debug = protectedView.owningInstance.debug;
        debug.sharedObjectLifeCycleChecks.markReadyForFinalize(instance.sharedObject);
        debug.protectedViews
            .getValue(instance)
            .invalidate();
        debug.protectedViews.deleteValue(instance);
        instance.debugOnAllocate = () => undefined;
        debug.onAllocate.removeListener(instance);
        if (!instance.sharedObject.isStatic) {
            const type = instance.sharedObject.isStatic ? "static" : "instance";
            const address = (0, number_get_hex_string_js_1.numberGetHexString)(instance.sharedObject.getPtr());
            const label = (0, string_normalize_null_undefined_to_empty_js_1.stringNormalizeNullUndefinedToEmpty)(_debug_js_1._Debug.label);
            _debug_js_1._Debug.verboseLog(["WASM", "MEMORY"], `released (${type}) ${nameOfInstance} ${address} - ${label}`);
        }
    }
}
exports.DebugSharedObjectChecks = DebugSharedObjectChecks;
//# sourceMappingURL=debug-shared-object-checks.js.map