import { Mat2 } from "../../mat2/mat2.js";
import { TTypedArray } from "../../t-typed-array.js";
import { ITypedArrayTupleFactory } from "../../i-typed-array-tuple-factory.js";
import { IReadonlyVec2, Vec2 } from "../../vec2/vec2.js";
import { IReadonlyMat3, Mat3 } from "../../mat3/mat3.js";
import { TTypedArrayCtor } from "../../t-typed-array-ctor.js";
import { TPickExcept } from "../../../../typescript/t-pick-except.js";
import { TTypedArrayTupleMutativeMethods } from "../../a-typed-array-tuple.js";
/**
 * @public
 */
export type TRange2dCtorArgs = [xMin: number, xMax: number, yMin: number, yMax: number];
/**
 * @public
 * Constructor for {@link Range2d}.
 */
export interface IRange2dCtor<TArray extends TTypedArray> {
    /**
     * The size in bytes of each element in the array.
     */
    readonly BYTES_PER_ELEMENT: number;
    readonly prototype: Range2d<TArray>;
    readonly factory: ITypedArrayTupleFactory<Range2d<TArray>, TRange2dCtorArgs>;
    new (): Range2d<TArray>;
}
/**
 * @public
 * {@link Range2d}.
 */
export interface IReadonlyRange2d<TArray extends TTypedArray> extends TPickExcept<Readonly<Range2d<TArray>>, "setXMin" | "setXMax" | "setYMin" | "setYMax" | "update" | "bound" | "ensureAABB" | "ensureMinRange" | "translateBy" | "copyFromBuffer" | TTypedArrayTupleMutativeMethods> {
}
/**
 * @public
 * 2x2 matrix representing a 2d range.
 *
 * @remarks
 * See static properties for constructors. Instances are not an extension of this class, but of the static members.
 */
export declare abstract class Range2d<TArray extends TTypedArray> extends Mat2<TArray> {
    static f64: IRange2dCtor<Float64Array>;
    static f32: IRange2dCtor<Float32Array>;
    static u32: IRange2dCtor<Uint32Array>;
    static i32: IRange2dCtor<Int32Array>;
    static u16: IRange2dCtor<Uint16Array>;
    static i16: IRange2dCtor<Int16Array>;
    static u8c: IRange2dCtor<Uint8ClampedArray>;
    static u8: IRange2dCtor<Uint8Array>;
    static i8: IRange2dCtor<Int8Array>;
    static getCtor<TCtor extends TTypedArrayCtor>(ctor: TCtor): IRange2dCtor<InstanceType<TCtor>>;
    protected static constructors: Map<TTypedArrayCtor, Function>;
    ["constructor"]: IRange2dCtor<TArray>;
    /**
     * xMin
     */
    0: number;
    /**
     * xMax
     */
    1: number;
    /**
     * yMin
     */
    2: number;
    /**
     * yMax
     */
    3: number;
    setXMin(_value: number): void;
    setXMax(_value: number): void;
    setYMin(_value: number): void;
    setYMax(_value: number): void;
    getXMin(): number;
    getXMax(): number;
    getYMin(): number;
    getYMax(): number;
    update(..._args: TRange2dCtorArgs): void;
    getRange<TResult extends TTypedArray = TArray>(_result?: Vec2<TResult>): Vec2<TResult>;
    getXRange(): number;
    getYRange(): number;
    getXMaxAbs(): number;
    getYMaxAbs(): number;
    getCenter<TResult extends TTypedArray = TArray>(_result?: Vec2<TResult>): Vec2<TResult>;
    getXCenter(): number;
    getYCenter(): number;
    mat3Multiply<TResult extends TTypedArray = TArray>(_mat: IReadonlyMat3<TTypedArray>, _writeTo?: Range2d<TResult>): Range2d<TResult>;
    unionRange<TResult extends TTypedArray = TArray>(_range: IReadonlyRange2d<TTypedArray>, _writeTo?: Range2d<TResult>): Range2d<TResult>;
    extendRange<TResult extends TTypedArray = TArray>(_x: number, _y: number, _writeTo?: Range2d<TResult>): Range2d<TResult>;
    /**
     * Creates a transform matrix that maps values in this range onto `_toRange`.
     */
    getRangeTransform<TResult extends TTypedArray = TArray>(_toRange: IReadonlyRange2d<TTypedArray>, _result?: IReadonlyMat3<TResult>): Mat3<TResult>;
    isPointInRange(_point: IReadonlyVec2<TTypedArray>): boolean;
    doesRangeIntersect(_range: IReadonlyRange2d<TArray>): boolean;
    containsRange(_range: IReadonlyRange2d<TArray>): boolean;
    /**
     * Scales the range relative to a point (may not be outside of the range).
     *
     * @remarks
     * If the point is at a boundary, then the range will be scaled such that that boundary is not changed. Where the
     * point is away from a boundary, the updated range will have boundaries proportional to the distance from the center
     * of the range.
     *
     * E.g. scaling factor of 0.5, P represents the position of the point in the range:
     *```
     * XMin                      XMax
     *  |P-------------------------|
     *  |-------------|
     *
     *  |------------P-------------|
     *        |-------------|
     *```
     */
    scaleRelativeTo<TResult extends TTypedArray = TArray>(_scalingFactor: number, _relativeTo: IReadonlyVec2<TTypedArray>, _result?: Range2d<TResult>): Range2d<TResult>;
    /**
     * Bound this range to the argument.
     *
     * @remarks
     * Where this range is larger than the bounding range, it will be resized to fit.
     * Where this range is smaller than the bounding range but not contained, it will be moved maintaining its size. It
     * Will be moved such that the the side furthest from the bounding range will be at the edge of the boundary.
     */
    bound(_boundTo: IReadonlyRange2d<TTypedArray>): void;
    ensureAABB(): void;
    /**
     * Bound this range to be at least as large as the argument.
     *
     * @remarks
     * Where this range is smaller than the bounding range, it will be resized to be the minimum. This is done by adding
     * equally to both the min and max.
     */
    ensureMinRange(_xMinRange: number, _yMinRange: number): void;
    /**
     * Shifts the range by (dx, dy).
     */
    translateBy(_dx: number, _dy: number): void;
    /**
     * If endianness is not supplied the platform's endianness will be used.
     */
    copyFromBuffer(_memoryDataView: DataView, _pointer: number, _littleEndian?: boolean): void;
    /**
     * If endianness is not supplied the platform's endianness will be used.
     */
    copyToBuffer(_memoryDataView: DataView, _pointer: number, _littleEndian?: boolean): void;
    /**
     * Although the typed array tuples extend a typed array, they are not structurally compatible.
     * This function returns the argument passed without modification but cast as the underlying storage type, e.g. Float32Array.
     */
    castToBaseType(): TArray;
    TTypeGuardRange2d: true;
}
/**
 * @public
 * Float32 {@link Range2d}.
 */
export type TF32Range2d = Range2d<Float32Array>;
/**
 * @public
 * Float64 {@link Range2d}.
 */
export type TF64Range2d = Range2d<Float64Array>;
