import { TTypedArray } from "../../t-typed-array.js";
import { ITypedArrayTupleFactory } from "../../i-typed-array-tuple-factory.js";
import { TTypedArrayCtor } from "../../t-typed-array-ctor.js";
import { Vec2 } from "../vec2.js";
import { Mat2 } from "../../mat2/mat2.js";
import { TPickExcept } from "../../../../typescript/t-pick-except.js";
import { TTypedArrayTupleMutativeMethods } from "../../a-typed-array-tuple.js";
/**
 * @public
 */
export type TRange1dCtorArgs = [min: number, max: number];
/**
 * @public
 * Constructor for {@link Range1d}.
 */
export interface IRange1dCtor<TArray extends TTypedArray> {
    /**
     * The size in bytes of each element in the array.
     */
    readonly BYTES_PER_ELEMENT: number;
    readonly prototype: Range1d<TArray>;
    readonly factory: ITypedArrayTupleFactory<Range1d<TArray>, TRange1dCtorArgs>;
    new (): Range1d<TArray>;
}
/**
 * @public
 * {@link Range1d}.
 */
export interface IReadonlyRange1d<TArray extends TTypedArray> extends TPickExcept<Readonly<Range1d<TArray>>, "setMin" | "setMax" | "bound1d" | "translate1d" | "copyFromBuffer" | TTypedArrayTupleMutativeMethods> {
}
/**
 * @public
 * Vec2 representing a 1d range.
 *
 * @remarks
 * See static properties for constructors. Instances are not an extension of this class, but of the static members.
 */
export declare abstract class Range1d<TArray extends TTypedArray> extends Vec2<TArray> {
    static f64: IRange1dCtor<Float64Array>;
    static f32: IRange1dCtor<Float32Array>;
    static u32: IRange1dCtor<Uint32Array>;
    static i32: IRange1dCtor<Int32Array>;
    static u16: IRange1dCtor<Uint16Array>;
    static i16: IRange1dCtor<Int16Array>;
    static u8c: IRange1dCtor<Uint8ClampedArray>;
    static u8: IRange1dCtor<Uint8Array>;
    static i8: IRange1dCtor<Int8Array>;
    static getCtor<TCtor extends TTypedArrayCtor>(ctor: TCtor): IRange1dCtor<InstanceType<TCtor>>;
    protected static constructors: Map<TTypedArrayCtor, Function>;
    ["constructor"]: IRange1dCtor<TArray>;
    /**
     * min
     */
    0: number;
    /**
     * max
     */
    1: number;
    setMin(_value: number): void;
    setMax(_value: number): void;
    getMin(): number;
    getMax(): number;
    getRange(): number;
    getCenter(): number;
    mat2Multiply<TResult extends TTypedArray = TArray>(_mat: Readonly<Mat2<TTypedArray>>, _writeTo?: Range1d<TResult>): Range1d<TResult>;
    unionRange<TResult extends TTypedArray = TArray>(_range: Readonly<Range1d<TTypedArray>>, _writeTo?: Range1d<TResult>): Range1d<TResult>;
    extendRange<TResult extends TTypedArray = TArray>(_value: number, _writeTo?: Range1d<TResult>): Range1d<TResult>;
    /**
     * Creates a transform matrix that maps values in this range onto `_toRange`.
     */
    getRangeTransform<TResult extends TTypedArray = TArray>(_toRange: Readonly<Range1d<TTypedArray>>, _result?: Mat2<TResult>): Mat2<TResult>;
    isValueInRange1d(_value: number): boolean;
    /**
     * Scales the range relative to a point (may not be outside of the range).
     *
     * @remarks
     * If the point is at a boundary, then the range will be scaled such that that boundary is not changed. Where the
     * point is away from a boundary, the updated range will have boundaries proportional to the distance from the center
     * of the range.
     *
     * E.g. scaling factor of 0.5, P represents the position of the point in the range:
     *```
     *  min                       max
     *  |P-------------------------|
     *  |-------------|
     *
     *  |------------P-------------|
     *        |-------------|
     *```
     */
    scaleRelativeTo<TResult extends TTypedArray = TArray>(_scalingFactor: number, _relativeTo: number, _result?: Range1d<TResult>): Range1d<TResult>;
    /**
     * Bound this range to the argument.
     *
     * @remarks
     * Where this range is larger than the bounding range, it will be resized to fit.
     * Where this range is smaller than the bounding range but not contained, it will be moved maintaining its size. It
     * Will be moved such that the the side furthest from the bounding range will be at the edge of the boundary.
     */
    bound1d(_boundTo: Range1d<TTypedArray>): void;
    /**
     * Shifts the range by dv.
     */
    translate1d(_dv: number): void;
    /**
     * If endianness is not supplied the platform's endianness will be used.
     */
    copyFromBuffer(_memoryDataView: DataView, _pointer: number, _littleEndian?: boolean): void;
    /**
     * If endianness is not supplied the platform's endianness will be used.
     */
    copyToBuffer(_memoryDataView: DataView, _pointer: number, _littleEndian?: boolean): void;
    /**
     * Although the typed array tuples extend a typed array, they are not structurally compatible.
     * This function returns the argument passed without modification but cast as the underlying storage type, e.g. Float32Array.
     */
    castToBaseType(): TArray;
    TTypeGuardRange1d: true;
}
/**
 * @public
 * Float32 {@link Range1d}.
 */
export type TF32Range1d = Range1d<Float32Array>;
/**
 * @public
 * Float64 {@link Range1d}.
 */
export type TF64Range1d = Range1d<Float64Array>;
