import { getGlobal } from "../runtime/get-global.js";
import { arrayAddToSet } from "../array/impl/array-add-to-set.js";
import { _Set } from "../set/_set.js";
/**
 * @public
 * Utilities for debug builds.
 */
export class _Debug {
    static get label() {
        return _Debug._label;
    }
    static set label(label) {
        _Debug._label = label;
    }
    static applyLabel(label, callback) {
        _Debug.label = label;
        const ret = callback();
        _Debug.label = undefined;
        return ret;
    }
    /**
     * Most debuggers will ignore `debugger` statements in node_modules.
     * Skirt around this by letting the consumer set their own callback for this.
     *
     * @param onBreakpoint - called on debug assert etc, you should provide a function with a `debugger` statement.
     *
     * @example
     * ```typescript
     * // the braces are not optional
     * _Debug.configureBreakpoint(() => { debugger; })
     * ```
     */
    static configureBreakpoint(onBreakpoint) {
        _Debug.onBreakpoint = onBreakpoint;
    }
    /**
     * Convenience method to run multiple asserts.
     *
     * @returns A boolean value to make linting happy...
     *
     * @example
     * ```typescript
     * _BUILD.DEBUG && _Debug.runBlock(() => {
     *     _Debug.assert(someCondition, "someCondition was wrong");
     *     // ...
     * });
     * ```
     */
    static runBlock(cb) {
        cb();
        return true;
    }
    /**
     * Convenience method to run multiple asserts if flag set.
     *
     * @returns A boolean value to make linting happy...
     *
     * @remarks
     * Must still be hidden behind _BUILD.DEBUG check for dead code removal.
     *
     * @example
     * ```typescript
     * _BUILD.DEBUG && _Debug.conditionalBlock("SOME_FLAG", () => {
     *     _Debug.assert(someCondition, "someCondition was wrong");
     *     // ...
     * });
     * ```
     */
    static conditionalBlock(flag, cb) {
        if (this.isFlagSet(flag)) {
            cb();
        }
        return true;
    }
    /**
     * Throws an `Error` with the given message if the condition is false.
     *
     * @returns A boolean value to make linting happy...
     *
     * @example
     * ```typescript
     * function foo(a1: number) {
     *     // not suitable for a production check, the programmer lied about the input type they supplied
     *     _BUILD.DEBUG && _Debug.assert(a1 != null, "a1 must be supplied");
     * }
     * ```
     *
     * @remarks
     * If `_BUILD.DISABLE_BREAKPOINT_FLAG` is false or unset then a breakpoint will be hit first.
     *
     *
     * Debug asserts are useful for providing hints to the programmer that they aren't meeting the contract of the API.
     */
    static assert(condition, errorMessage) {
        if (!condition) {
            if (!_Debug.isFlagSet("DISABLE_BREAKPOINT")) {
                _Debug.breakpoint();
            }
            throw new Error(`assert fail: ${errorMessage}`);
        }
        return true;
    }
    /**
     * Throws an `Error` with the given message.
     * @returns A boolean value to make linting happy... will never return.
     *
     * @example
     * ```typescript
     * if (errorCondition) {
     *     // in debug mode we error
     *     _BUILD.DEBUG && _Debug.error("oopsy");
     *     // in production we fall back to some other behavior
     *     return errorConditionValue;
     * }
     * ```
     *
     * @remarks
     * If `_BUILD.DEBUG` is true and `_BUILD.DISABLE_BREAKPOINT` is false or unset then a breakpoint will be hit first.
     */
    static error(message) {
        if (!_Debug.isFlagSet("DISABLE_BREAKPOINT")) {
            _Debug.breakpoint();
        }
        throw new Error(message);
    }
    /**
     * Used in place of `debugger` statements when writing libraries. Should generally not be used directly.
     */
    static breakpoint() {
        _Debug.onBreakpoint();
        return true;
    }
    /**
     * Logging which can be conditionally enabled by setting either `VERBOSE` to true on build options (enabling everything),
     * or by calling setLoggingTags and specifying which tags you'd like enabled.
     *
     * @example
     * ```typescript
     * function foo(a1: number) {
     *     _BUILD.DEBUG && _Debug.verboseLog(`got me a ${a1}`);
     * }
     * ```
     */
    static verboseLog(tags, message, ancillaryObject) {
        arrayAddToSet(tags, _Debug._seenTags);
        if (!_Debug.isFlagSet("VERBOSE")) {
            if (!tags.some(tag => _Debug._enabledTags.has(tag))) {
                return;
            }
        }
        const prefixedMessage = tags.length > 0
            ? ["[", tags.join(", "), "] ", message].join("")
            : message;
        if (ancillaryObject == null) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-member-access
            console.debug(prefixedMessage);
        }
        else {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-member-access
            console.debug(prefixedMessage, ancillaryObject);
        }
    }
    static getStackTrace() {
        const error = new Error();
        let stack = error.stack;
        if (stack == null) {
            try {
                // noinspection ExceptionCaughtLocallyJS
                throw error;
            }
            catch (e) {
                stack = e.stack;
            }
        }
        return stack.toString();
    }
    /**
     * Used to set debug flags in an environment independent way.
     */
    static setFlag(flag, value) {
        var _a;
        var _b;
        const build = ((_a = (_b = getGlobal())["_BUILD"]) !== null && _a !== void 0 ? _a : (_b["_BUILD"] = {}));
        build[flag] = value;
    }
    /**
     * Used to get debug flags in an environment independent way.
     */
    static isFlagSet(flag) {
        var _a, _b;
        const build = ((_a = getGlobal()["_BUILD"]) !== null && _a !== void 0 ? _a : {});
        return (_b = build[flag]) !== null && _b !== void 0 ? _b : false;
    }
    static setLoggingTags(tags) {
        _Debug._enabledTags = new Set(tags);
    }
    /**
     * @returns The tags that have been seen so far. Tags will be added as `verboseLog` is called....
     */
    static getTags() {
        return _Set.valuesToArray(this._seenTags);
    }
}
_Debug.onBreakpoint = () => {
    // eslint-disable-next-line no-debugger
    debugger;
};
_Debug._label = undefined;
_Debug._enabledTags = new Set();
_Debug._seenTags = new Set();
//# sourceMappingURL=_debug.js.map