import { _Debug } from "../debug/_debug.js";
import { LinkedReferences } from "./linked-references.js";
import { TemporaryListener } from "./temporary-listener.js";
/**
 * @public
 * Provides a way to handle cleanup of manually managed resources where there is not a single owner.
 * NB The object is pre-claimed (ref count 1) on creation.
 */
export class AReferenceCounted {
    constructor() {
        this.references = 1;
        this.linkedReferences = null;
        this.onFreeListener = null;
    }
    /**
     * Take a claim on the object, preventing it from being destroyed. Once you're done with the object you should
     * call `release`.
     */
    claim() {
        if (this.references === 0) {
            _BUILD.DEBUG && _Debug.error("object has been released already");
            return;
        }
        ++this.references;
    }
    /**
     * Release claim on the object, objects start with a reference count of 1 (the creator); if you didn't create the
     * object, you shouldn't release it if you didn't call `claim` first.
     */
    release() {
        if (--this.references === 0) {
            this.onFree();
        }
    }
    getIsDestroyed() {
        return this.references <= 0;
    }
    getLinkedReferences() {
        if (this.linkedReferences == null) {
            return this.linkedReferences = new LinkedReferences(this);
        }
        return this.linkedReferences;
    }
    registerOnFreeListener(callback) {
        var _a;
        this.onFreeListener = (_a = this.onFreeListener) !== null && _a !== void 0 ? _a : new TemporaryListener();
        this.onFreeListener.addListener(callback);
    }
    unregisterOnFreeListener(callback) {
        if (this.onFreeListener) {
            this.onFreeListener.removeListener(callback);
        }
    }
    /**
     * DO NOT CALL THIS DIRECTLY, CALL RELEASE.
     */
    onFree() {
        var _a, _b;
        (_a = this.linkedReferences) === null || _a === void 0 ? void 0 : _a.unlinkAllRefs();
        (_b = this.onFreeListener) === null || _b === void 0 ? void 0 : _b.clearingEmit();
    }
}
//# sourceMappingURL=a-reference-counted.js.map