import { __awaiter } from "tslib";
import { getWasmTestMemory } from "../util/get-wasm-test-memory.js";
import { getEmscriptenWrapper } from "./get-emscripten-wrapper.js";
import { _Fp } from "../../fp/_fp.js";
import { _Production } from "../../production/_production.js";
import { Test_resetLifeCycle } from "../../test-util/test_reset-life-cycle.js";
/**
 * @public
 */
export const emscriptenAsanTestModuleOptions = {
    disabledErrors: new Set(["==42==WARNING: AddressSanitizer failed to allocate 0xfffffffc bytes"]),
    initialMemoryPages: 8192,
    maxMemoryPages: 8192,
    quitThrowsWith: {},
};
/**
 * @public
 */
export const emscriptenSafeHeapTestModuleOptions = {
    disabledErrors: new Set(),
    initialMemoryPages: 128,
    maxMemoryPages: 512,
    quitThrowsWith: {},
};
/**
 * @public
 * A wrapper for running emscripten modules built with ASAN. To see the full report, you must call `endEmscriptenProgram`.
 * @typeParam TEmscriptenBindings - The generated WASM bindings.
 * @typeParam TWrapperExtensions - Extensions to the test wrapper itself, i.e. overwrite the module with test specific logic.
 */
export class SanitizedEmscriptenTestModule {
    constructor(testModule, options, extension) {
        this.testModule = testModule;
        this.options = options;
        this.extension = extension;
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            const memory = getWasmTestMemory({ initial: this.options.initialMemoryPages, maximum: this.options.maxMemoryPages });
            this._wrapper = (yield getEmscriptenWrapper(memory, this.testModule, Object.assign({ ASAN_OPTIONS: "allocator_may_return_null=1", print: _Fp.noOp, printErr: (error) => {
                    // looks like asan writes to stderr regardless of option...
                    if (this.options.disabledErrors.has(error)) {
                        return;
                    }
                    throw new Error(error);
                }, quit: () => {
                    // emscripten hits asserts if quit doesn't interrupt execution
                    // the default behavior in node is to kill the process which would kill the tests
                    // by throwing something unique we can catch but avoid swallowing actual errors
                    throw this.options.quitThrowsWith;
                } }, this.extension)));
        });
    }
    get wrapper() {
        if (this._wrapper == null) {
            throw _Production.createError("initialize must be called first");
        }
        return this._wrapper;
    }
    /**
     * NB you MUST set the flag `-s EXIT_RUNTIME=1` for this to work.
     */
    endEmscriptenProgram() {
        // kick off asan checks
        try {
            this.wrapper.instance._jsUtilEndProgram(0);
        }
        catch (error) {
            if (error !== emscriptenAsanTestModuleOptions.quitThrowsWith) {
                throw error;
            }
        }
    }
    /**
     * Call this before each test case.
     */
    reset() {
        Test_resetLifeCycle();
        if (this._wrapper) {
            this._wrapper.debug.uniquePointers.clear();
        }
    }
}
//# sourceMappingURL=sanitized-emscripten-test-module.js.map