import { ReferenceCountedPtr } from "../util/reference-counted-ptr.js";
import { _Debug } from "../../debug/_debug.js";
import { DebugProtectedView } from "../../debug/debug-protected-view.js";
import { _Production } from "../../production/_production.js";
import { nullPointer } from "../emscripten/null-pointer.js";
import { DebugSharedObjectChecks } from "../util/debug-shared-object-checks.js";
/**
 * @public
 * Typed array shared between wasm and javascript.
 */
export class SharedArray {
    static createOneF32(wrapper, bindToReference, length, clearMemory = false, allocationFailThrows = true) {
        return SharedArray.createOne("f32SharedArray", wrapper, bindToReference, Float32Array, length, clearMemory, allocationFailThrows);
    }
    static createOneF64(wrapper, bindToReference, length, clearMemory = false, allocationFailThrows = true) {
        return SharedArray.createOne("f64SharedArray", wrapper, bindToReference, Float64Array, length, clearMemory, allocationFailThrows);
    }
    static createOne(prefix, wrapper, bindToReference, ctor, length, clearMemory = false, allocationFailThrows = true) {
        const ptr = SharedArray.allocateMemory(prefix, wrapper, length, clearMemory, allocationFailThrows);
        if (ptr !== nullPointer) {
            const sa = new SharedArray(prefix, ctor, wrapper, length, ptr);
            bindToReference === null || bindToReference === void 0 ? void 0 : bindToReference.linkRef(sa.sharedObject);
            return sa;
        }
        return null;
    }
    static allocateMemory(cMethodPrefix, wrapper, length, clearMemory, allocationFailThrows) {
        const ptr = wrapper.instance[`_${cMethodPrefix}_createOne`](length, clearMemory);
        if (ptr === nullPointer && allocationFailThrows) {
            throw _Production.createError("Failed to allocate memory for shared array.");
        }
        return ptr;
    }
    getInstance() {
        if (_BUILD.DEBUG) {
            _Debug.assert(!this.sharedObject.getIsDestroyed(), "access violation - object has been destroyed");
            return this.wrapper.debug.protectedViews
                .getValue(this)
                .createProtectedView(this.instance);
        }
        else {
            return this.instance;
        }
    }
    constructor(cMethodPrefix, ctor, wrapper, length, pointer) {
        this.onMemoryResize = () => {
            this.instance = this.createLocalInstance();
        };
        this.sharedObject = new ReferenceCountedPtr(false, pointer, wrapper);
        this.sharedObject.registerOnFreeListener(() => this.wrapper.instance[this.cDelete](this.sharedObject.getPtr()));
        this.sharedObject.registerOnFreeListener(wrapper.memoryResize.addTemporaryListener(this));
        this.length = length;
        this.ctor = ctor;
        this.cDelete = `_${cMethodPrefix}_delete`;
        this.cGetArrayAddress = `_${cMethodPrefix}_getArrayAddress`;
        this.wrapper = wrapper;
        this.elementByteSize = ctor.BYTES_PER_ELEMENT;
        _BUILD.DEBUG && _Debug.runBlock(() => {
            const protectedView = DebugProtectedView.createTypedArrayView(this.wrapper);
            DebugSharedObjectChecks.registerWithCleanup(this, protectedView, "shared array");
        });
        this.instance = this.createLocalInstance();
    }
    createLocalInstance() {
        const arrayPtr = this.wrapper.instance[this.cGetArrayAddress](this.sharedObject.getPtr());
        _BUILD.DEBUG && _Debug.assert(arrayPtr !== nullPointer, "failed to get array address");
        return new this.ctor(this.wrapper.memory.buffer, arrayPtr, this.length);
    }
}
//# sourceMappingURL=shared-array.js.map