import { ReferenceCountedPtr } from "../util/reference-counted-ptr.js";
import { nullPointer } from "../emscripten/null-pointer.js";
import { _Production } from "../../production/_production.js";
import { DebugProtectedView } from "../../debug/debug-protected-view.js";
import { _Debug } from "../../debug/_debug.js";
import { DebugSharedObjectChecks } from "../util/debug-shared-object-checks.js";
import { _Number } from "../../number/_number.js";
/**
 * @public
 * {@inheritDoc ISharedMemoryBlock}
 */
export class SharedMemoryBlock {
    static createOne(wrapper, bindToReference, byteSize, allocationFailThrows = true) {
        _BUILD.DEBUG && wrapper.debug.onAllocate.emit();
        const pointer = wrapper.instance._jsUtilMalloc(byteSize);
        if (pointer == nullPointer) {
            if (allocationFailThrows) {
                throw _Production.createError("Failed to allocate memory for shared memory block.");
            }
            else {
                return null;
            }
        }
        const smb = new SharedMemoryBlock(wrapper, pointer, byteSize);
        bindToReference === null || bindToReference === void 0 ? void 0 : bindToReference.linkRef(smb.sharedObject);
        return smb;
    }
    getDataView() {
        if (_BUILD.DEBUG) {
            return this.wrapper.debug.protectedViews
                .getValue(this)
                .createProtectedView(this.dataView);
        }
        else {
            return this.dataView;
        }
    }
    constructor(wrapper, pointer, byteSize) {
        this.wrapper = wrapper;
        this.onMemoryResize = () => {
            if (this.wrapper == null) {
                _BUILD.DEBUG && _Debug.error("object has been destroyed");
                return;
            }
            this.dataView = this.recreateDataView();
        };
        this.pointer = pointer;
        this.byteSize = byteSize;
        this.sharedObject = new ReferenceCountedPtr(false, pointer, wrapper);
        this.sharedObject.registerOnFreeListener(this.wrapper.memoryResize.addTemporaryListener(this));
        this.sharedObject.registerOnFreeListener(() => this.wrapper.instance._jsUtilFree(this.pointer));
        _BUILD.DEBUG && _Debug.runBlock(() => {
            const protectedView = new DebugProtectedView(this.wrapper, `SharedMemoryBlock - memory resize danger: don't hold reference to the DataView ${_Number.getHexString(this.pointer)}`);
            DebugSharedObjectChecks.registerWithCleanup(this, protectedView, "SharedMemoryBlock");
        });
        this.dataView = this.recreateDataView();
    }
    recreateDataView() {
        return new DataView(this.wrapper.memory.buffer, this.pointer, this.byteSize);
    }
}
//# sourceMappingURL=shared-memory-block.js.map